@@ -22,10 +22,11 @@ import subprocess
 import tempfile
 import textwrap
 
-from awscli.compat import shlex_quote, urlopen
+from botocore.exceptions import ClientError
+
+from awscli.compat import shlex_quote, urlopen, ensure_text_type
 from awscli.customizations.commands import BasicCommand
 from awscli.customizations.utils import create_client_from_parsed_globals
-from awscli.errorhandler import ClientError
 
 
 LOG = logging.getLogger(__name__)
@@ -35,7 +36,7 @@ IAM_USER_POLICY_TIMEOUT = datetime.timedelta(minutes=15)
 IAM_PATH = '/AWS/OpsWorks/'
 
 HOSTNAME_RE = re.compile(r"^(?!-)[a-z0-9-]{1,63}(?<!-)$", re.I)
-INSTANCE_ID_RE = re.compile(r"^i-[0-9a-f]{8}$")
+INSTANCE_ID_RE = re.compile(r"^i-[0-9a-f]+$")
 IP_ADDRESS_RE = re.compile(r"^\d+\.\d+\.\d+\.\d+$")
 
 IDENTITY_URL = \
@@ -110,6 +111,9 @@ class OpsWorksRegister(BasicCommand):
          'help_text': """If given, instead of a remote machine, the local
                          machine will be imported. Cannot be used together
                          with `target`."""},
+        {'name': 'use-instance-profile', 'action': 'store_true',
+         'help_text': """Use the instance profile instead of creating an IAM
+                         user."""},
         {'name': 'target', 'positional_arg': True, 'nargs': '?',
          'synopsis': '[<target>]',
          'help_text': """Either the EC2 instance ID or the hostname of the
@@ -125,6 +129,7 @@ class OpsWorksRegister(BasicCommand):
         self._use_address = None
         self._use_hostname = None
         self._name_for_iam = None
+        self.access_key = None
 
     def _create_clients(self, args, parsed_globals):
         self.iam = self._session.create_client('iam')
@@ -138,7 +143,7 @@ class OpsWorksRegister(BasicCommand):
         self.retrieve_stack(args)
         self.validate_arguments(args)
         self.determine_details(args)
-        self.create_iam_entities()
+        self.create_iam_entities(args)
         self.setup_target_machine(args)
 
     def prevalidate_arguments(self, args):
@@ -168,6 +173,11 @@ class OpsWorksRegister(BasicCommand):
                 raise ValueError(
                     "--override-public-ip is not supported for EC2.")
 
+        if args.infrastructure_class == 'on-premises' and \
+                args.use_instance_profile:
+            raise ValueError(
+                "--use-instance-profile is only supported for EC2.")
+
         if args.hostname:
             if not HOSTNAME_RE.match(args.hostname):
                 raise ValueError(
@@ -268,7 +278,8 @@ class OpsWorksRegister(BasicCommand):
 
         if args.infrastructure_class == 'ec2' and args.local:
             # make sure the regions match
-            region = json.loads(urlopen(IDENTITY_URL).read())['region']
+            region = json.loads(
+                ensure_text_type(urlopen(IDENTITY_URL).read()))['region']
             if region != self._stack['Region']:
                 raise ValueError(
                     "The stack's and the instance's region must match.")
@@ -312,20 +323,25 @@ class OpsWorksRegister(BasicCommand):
             self._use_hostname = None
             self._name_for_iam = args.target
 
-    def create_iam_entities(self):
+    def create_iam_entities(self, args):
         """
         Creates an IAM group, user and corresponding credentials.
 
         Provides `self.access_key`.
         """
 
+        if args.use_instance_profile:
+            LOG.debug("Skipping IAM entity creation")
+            self.access_key = None
+            return
+
         LOG.debug("Creating the IAM group if necessary")
         group_name = "OpsWorks-%s" % clean_for_iam(self._stack['StackId'])
         try:
             self.iam.create_group(GroupName=group_name, Path=IAM_PATH)
             LOG.debug("Created IAM group %s", group_name)
         except ClientError as e:
-            if e.error_code == 'EntityAlreadyExists':
+            if e.response.get('Error', {}).get('Code') == 'EntityAlreadyExists':
                 LOG.debug("IAM group %s exists, continuing", group_name)
                 # group already exists, good
                 pass
@@ -343,7 +359,7 @@ class OpsWorksRegister(BasicCommand):
             try:
                 self.iam.create_user(UserName=username, Path=IAM_PATH)
             except ClientError as e:
-                if e.error_code == 'EntityAlreadyExists':
+                if e.response.get('Error', {}).get('Code') == 'EntityAlreadyExists':
                     LOG.debug(
                         "IAM user %s already exists, trying another name",
                         username
@@ -436,11 +452,13 @@ class OpsWorksRegister(BasicCommand):
 
     def _pre_config_document(self, args):
         parameters = dict(
-            access_key_id=self.access_key['AccessKeyId'],
-            secret_access_key=self.access_key['SecretAccessKey'],
             stack_id=self._stack['StackId'],
             **self._prov_params["Parameters"]
         )
+        if self.access_key:
+            parameters['access_key_id'] = self.access_key['AccessKeyId']
+            parameters['secret_access_key'] = \
+                self.access_key['SecretAccessKey']
         if self._use_hostname:
             parameters['hostname'] = self._use_hostname
         if args.private_ip:
